"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class AggregateSourceElements extends Map {
    constructor(entries) {
        super(entries);
    }
    /**
     * DO NOT USE. Use getSourceElement instead.
     * Returns the FIRST matching AggregateSourceElement. We should not use this method
     * because you're not guaranteed to get the source element from the correct package.
     */
    findSourceElementByKey(key) {
        let returnValue;
        this.forEach(aggregateSourceElements => {
            if (aggregateSourceElements.has(key)) {
                returnValue = aggregateSourceElements.get(key);
            }
        });
        return returnValue;
    }
    /**
     * Returns `AggregateSourceElements` from all package directories matching the
     * given source element key.
     *
     * @param key a metadata type name pair.  E.g., ApexClass__MyClass
     */
    filterSourceElementsByKey(key) {
        const filteredElements = new AggregateSourceElements();
        this.forEach(aseMap => {
            if (aseMap.has(key)) {
                const ase = aseMap.get(key);
                filteredElements.setIn(ase.getPackageName(), ase.getKey(), ase);
            }
        });
        return filteredElements;
    }
    /**
     * Attempts to find the parent AggregateSourceElement across package directories
     * that contains the matching child metadata type and name as a WorkspaceElement.
     * It then returns the parent ASE with all WorkspaceElements removed except for
     * the matching child WorkspaceElement.  This allows fine-grained deploys/retrieves.
     *
     * E.g., Given the custom field "FieldA" that belongs to custom object "CO1";
     *         - CO1 has meta-xml file and "FieldB" in package directory X
     *         - CO1 also has "FieldA" and "FieldC" in package directory Y
     *       This method will return the "CO1" ASE from package dir Y containing
     *       only the "FieldA" WorkspaceElement.
     */
    findParentElement(parentKey, childType, childName) {
        let returnValue;
        // debug(`finding parent element: ${parentKey} containing child: ${childKey}`);
        this.forEach(aggregateSourceElements => {
            if (aggregateSourceElements.has(parentKey)) {
                const ase = aggregateSourceElements.get(parentKey);
                if (childType && childName) {
                    let matchingWorkspaceElement;
                    // Match the childType and childName with the workspace elements
                    const matchFound = ase.getWorkspaceElements().some(we => {
                        if (we.getFullName() === childName && we.getMetadataName() === childType) {
                            matchingWorkspaceElement = we;
                            return true;
                        }
                        return false;
                    });
                    if (matchFound) {
                        // debug('Found matching source element:', matchingWorkspaceElement.toObject(), 'in package:', ase.getPackageName());
                        // Set the workspace elements for the ASE to be the one matched
                        ase.workspaceElements = [matchingWorkspaceElement];
                        returnValue = ase;
                    }
                }
            }
        });
        return returnValue;
    }
    getSourceElement(packageName, key) {
        if (this.has(packageName)) {
            return this.get(packageName).get(key);
        }
    }
    deleteSourceElement(packageName, key) {
        if (this.has(packageName)) {
            this.get(packageName).delete(key);
        }
        return this;
    }
    setIn(packageName, sourceElementKey, sourceElement) {
        if (this.has(packageName)) {
            this.get(packageName).set(sourceElementKey, sourceElement);
        }
        else {
            this.set(packageName, new Map().set(sourceElementKey, sourceElement));
        }
        return this;
    }
    /**
     * Returns a flat array of all source elements across all packages
     */
    getAllSourceElements() {
        let elements = [];
        this.forEach(sourceElements => {
            elements = elements.concat([...sourceElements.values()]);
        });
        return elements;
    }
    getAllWorkspaceElements() {
        let elements = [];
        this.forEach(sourceElements => {
            [...sourceElements.values()].forEach(el => {
                elements = elements.concat(el.workspaceElements);
            });
        });
        return elements;
    }
    getAllSourceKeys() {
        let keys = [];
        this.forEach(sourceElements => {
            keys = keys.concat([...sourceElements.keys()]);
        });
        return keys;
    }
    isEmpty() {
        return this.size === 0;
    }
    merge(aggregateSourceElements) {
        aggregateSourceElements.forEach((aseMap, packageName) => {
            if (this.has(packageName)) {
                const elements = this.get(packageName);
                this.set(packageName, new Map([...elements, ...aseMap]));
            }
            else {
                this.set(packageName, aseMap);
            }
        });
        return this;
    }
}
exports.AggregateSourceElements = AggregateSourceElements;

//# sourceMappingURL=aggregateSourceElements.js.map
