"use strict";
/*
 * Copyright (c) 2018, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: BSD-3-Clause
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const SourceUtil = require("./sourceUtil");
const metadataTypeFactory_1 = require("./metadataTypeFactory");
const core_1 = require("@salesforce/core");
const path = require("path");
const aggregateSourceElements_1 = require("./aggregateSourceElements");
const MetadataRegistry = require("./metadataRegistry");
class SourceElementsResolver {
    constructor(org, sourceWorkSpaceAdapter) {
        this.sourceWorkSpaceAdapter = sourceWorkSpaceAdapter;
        this.org = org;
    }
    /**
     *
     * @param manifestPath - path to package.xml
     * @return {Map} aggregateSourceElements
     */
    async getSourceElementsFromManifest(manifestPath) {
        const typeNamePairs = await SourceUtil.parseToManifestEntriesArray(manifestPath);
        const sourceElements = await this.sourceWorkSpaceAdapter.getAggregateSourceElements(false);
        return this.parseTypeNamePairs(typeNamePairs, sourceElements);
    }
    /**
     * Filters the elements based on the manifest file.
     * @param typeNamePairs - type name pairs from the manifest file
     * @param sourceElements
     * @return {Map} aggregateSoureElements
     */
    parseTypeNamePairs(typeNamePairs, sourceElements) {
        let aggregateSourceElements = new aggregateSourceElements_1.AggregateSourceElements();
        typeNamePairs.forEach((entry) => {
            if (entry.name.includes('*')) {
                let keyMetadataType = entry.type;
                const metadataType = metadataTypeFactory_1.MetadataTypeFactory.getMetadataTypeFromMetadataName(keyMetadataType, this.sourceWorkSpaceAdapter.metadataRegistry);
                if (!metadataType) {
                    throw core_1.SfdxError.create('salesforce-alm', 'source', 'UnsupportedType', [keyMetadataType]);
                }
                if (metadataType.getMetadataName() !== metadataType.getAggregateMetadataName()) {
                    // In this case we are dealing with a decomposed item, so reload using the parent name
                    keyMetadataType = metadataType.getAggregateMetadataName();
                }
                sourceElements.forEach(elements => {
                    const wildcards = [...elements.keys()]
                        .filter(item => item.includes(keyMetadataType))
                        .map((typeAnyName) => {
                        let [mdType, ...rest] = typeAnyName.split('__');
                        const mdName = rest.join('__');
                        return {
                            type: mdType,
                            name: mdName
                        };
                    });
                    aggregateSourceElements.merge(this.parseTypeNamePairs(wildcards, sourceElements));
                });
            }
            else {
                //
                // We can't call SourceUtil.loadSourceElement() here because we have to match the key across
                // package directories.  E.g., a custom object can be defined in multiple package directories
                // and we need to return all matches from all package dirs based on a typeNamePair (i.e., key).
                //
                aggregateSourceElements.merge(sourceElements.filterSourceElementsByKey(MetadataRegistry.getMetadataKey(entry.type, entry.name)));
            }
        });
        return aggregateSourceElements;
    }
    /**
     *
     * @param options - any{}
     * @param aggregateSourceElements - any empty array of type aggregateSoureElements
     * @param tmpOutputDir
     */
    async getSourceElementsFromMetadata(options, aggregateSourceElements, tmpOutputDir) {
        tmpOutputDir = tmpOutputDir || (await SourceUtil.createOutputDir('decomposition'));
        const manifestPath = await SourceUtil.toManifest(this.org, options, tmpOutputDir);
        const isPathABundleError = path.extname(options.metadata).length > 0 && SourceUtil.containsMdBundle(options);
        if (isPathABundleError) {
            throw core_1.SfdxError.create('salesforce-alm', 'source', 'SourcePathInvalid', [options.metadata]);
        }
        if (manifestPath) {
            aggregateSourceElements = await this.getSourceElementsFromManifest(manifestPath);
        }
        else {
            throw core_1.SfdxError.create('salesforce-alm', 'source', 'failedToCreateManifest');
        }
        return aggregateSourceElements;
    }
}
exports.SourceElementsResolver = SourceElementsResolver;

//# sourceMappingURL=sourceElementsResolver.js.map
